from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible import constants as C
from ansible.plugins.callback import CallbackBase
from ansible.utils.color import colorize, hostcolor
import sys

class CallbackModule(CallbackBase):

    '''
    This is the nagios callback, which prints playbook results in nagios
    conform output
    '''

    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = 'stdout'
    CALLBACK_NAME = 'nagios'
    CALLBACK_WHITELIST = True

    def __init__(self):
        super(CallbackModule, self).__init__()
        self.detailed_results = []

    def v2_playbook_on_no_hosts_matched(self):
        self._display.display("UNKNOWN - no hosts matched ")
	sys.exit(3)

    def v2_runner_on_unreachable(self, result):
	pass

    def v2_runner_on_no_host(self, task):
	pass

    def v2_runner_on_failed(self, result, ignore_errors=False):
        delegated_vars = result._result.get('_ansible_delegated_vars', None)
        if 'exception' in result._result:
            if self._display.verbosity < 3:
                # extract just the actual error message from the exception text
                error = result._result['exception'].strip().split('\n')[-1]
                msg = "    An exception occurred during task execution. To see the full traceback, use -vvv. The error was: %s" % error
            else:
                msg = "    An exception occurred during task execution. The full traceback is:\n" + result._result['exception']

            self.detailed_results.append(msg)

            # finally, remove the exception from the result so it's not shown every time
            del result._result['exception']

        if not result._task.ignore_errors:
            if result._task.loop and 'results' in result._result:
                self._process_items(result)

            else:
                if delegated_vars:
                    self.detailed_results.append("    FAILED: [%s -> %s]: %s" % (result._host.get_name(), delegated_vars['ansible_host'], self._dump_results(result._result)))
                else:
                    self.detailed_results.append("    FAILED: [%s]: %s" % (result._host.get_name(), self._dump_results(result._result)))
        sys.exit(1)

    def v2_runner_on_unreachable(self, result):
        delegated_vars = result._result.get('_ansible_delegated_vars', None)
        if delegated_vars:
            self._display.display("CRITICAL - %s -> %s is unrechable => %s" % (result._host.get_name(), delegated_vars['ansible_host'], self._dump_results(result._result)))
        else:
            self._display.display("CRITICAL - %s is unreachable => %s" % (result._host.get_name(), self._dump_results(result._result)))
	sys.exit(2)

    def v2_playbook_on_stats(self, stats):
        try:
            host = stats.processed.keys()[0]
	except:
            return

	t = stats.summarize(host)

        if t['unreachable'] > 0 or t['failures'] > 0:
            self._display.display(u"CRITICAL - %s : ok=%s changed=%s unreachable=%s failed=%s" % (
                host,t['ok'],t['changed'],t['unreachable'],t['failures']),
                screen_only=True
            )
            for result in self.detailed_results:
                print(result)
	    sys.exit(2)
	elif t['changed'] > 1:
            self._display.display(u"WARNING - %s : ok=%s changed=%s unreachable=%s failed=%s" % (
                host,t['ok'],t['changed'],t['unreachable'],t['failures']),
                screen_only=True
            )
            sys.exit(1)
        else:
	    self._display.display(u"OK - %s tasks run successfully" % (t['ok']),screen_only=True)
